
# q-expansion of newform 108.6.a.c, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 108
weight = 6
poly_data = [-10, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-9, 18], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [0, -1], [-16, -3], [243, 8], [104, 12], [972, -20], [-316, -6], [3402, 4], [5832, 38], [1664, -81], [-4978, 60], [6804, -58], [2480, 162], [9234, -264], [-5832, 341], [972, 376], [4088, -720], [45032, 450], [-22356, -788], [-60607, 504], [14384, 840], [-7533, 1752], [-89424, 358], [44471, 1272], [-101088, -1055], [131936, -1260], [92583, -1240], [-150226, -324], [-16524, 1350], [-139768, 2835], [261711, -952], [95256, 1670], [163976, 1500], [-40824, -2777], [-41416, 315], [-113212, -7128], [47924, 2544], [222102, 4176], [-268272, 3963], [35721, 3224], [363536, 7584], [-264384, 5812], [-590971, 2304], [-511272, -1643], [290024, -6909], [-23476, 5178], [-429160, -1296], [-308124, -1416], [-1192906, 4536], [272160, 12074], [-205578, 9396], [32366, -10056], [-474336, 8864], [1293732, -1632], [-486, -15236], [672624, -4138], [709856, 1557], [696848, 16200], [1806948, 12668], [-10036, -35802], [730944, -24254], [-365236, 20412], [-375678, 23612], [1528661, -12936], [1174176, 25873], [-1395160, -16422], [-1284718, 9960], [-462915, -4152], [600554, -12828], [1670868, -29950], [-1352538, -10248], [-49408, -25101], [767732, -43116], [-2124064, -24708], [-2530116, -3868], [-2482488, -8177], [-1192552, -17820], [-1671840, -48004], [-702265, 6432], [-821340, -29040], [3869696, 11832], [-2579202, 34096], [-1866859, 74520], [-707584, -27945], [774684, 77424], [4349700, -58374], [-651145, -77112], [-6837048, -35713], [2910848, 6105], [4930227, -57184], [5557410, 52296], [-1935280, -55392], [1279881, -149576], [1523672, 79218], [5937462, 55140], [-2313360, -15067], [628884, -11384], [-3156580, -13428], [5301692, 53448], [-4661188, -49356], [-1249992, 39583], [6434397, 117336], [2288088, -100292], [-4852948, -80544], [500030, 36504], [2441907, -129848], [-3888000, -167126], [3045762, -38696], [-1190533, -92664], [-8475520, 127968], [10498094, -76320], [6956604, -130534], [-12170068, -27432], [-2255104, 46437], [-9345780, 167630], [3553280, 177876], [6424920, -171048], [-11508480, -103901], [6103917, 103504], [-9818446, 167220], [-9894211, -12912], [-624024, 161434], [-1960524, -256080], [2100704, -103356], [-6213024, 75533], [189308, 266736], [-710532, -63264], [-3186040, 106677], [-2846674, 32724], [-5301916, 328212], [15240960, -164916], [9419288, 260115], [1223882, 68472], [-2109240, 224760], [-69583, -159792], [13452480, -2106], [-7035520, 232416], [17494056, -223893], [21978864, -226694], [6105836, -163134], [-9789984, 93214], [-9117904, 61347], [16990560, 84448], [504314, 319368], [-6443388, 294876], [14133854, -256392], [21433572, 186324], [19208840, -308208], [-14604300, 412432], [7548602, -504480], [-31433508, -250072], [2184524, -577086], [-592920, 198068], [14374748, 395706], [-26267814, -24000], [-17315800, 339471], [-12649608, 233244], [-885997, -151464], [-9288432, 422155], [130005, -271488], [14193144, 650148], [12869456, -115803], [-51366069, 80136], [4420656, 3790], [22851234, 81448], [12408992, 102285], [-36702976, -135108]]
